
-- create the statusbar for the inflight timer
function InFlight:CreateBar()
	local f = CreateFrame("Frame", "InFlightBar", UIParent)
	local db = self.db.profile
	f:Hide()
	if not db.x then
		f:SetPoint("CENTER", UIParent, "CENTER", 0, 200)
	else
		f:SetPoint("TOPLEFT", UIParent, "TOPLEFT", db.x, db.y)
	end
	f:SetMovable(true)
	f:EnableMouse(true)
	f:RegisterForDrag("LeftButton")
	f:SetScript("OnDragStart", function() if IsShiftKeyDown() then this:StartMoving() end end)
	f:SetScript("OnDragStop", function() 
		this:StopMovingOrSizing() 
		local a,b,c,d,e = this:GetPoint()
		if a == "TOPLEFT" and c == "TOPLEFT" then
			db.x = floor(d + 0.5)
			db.y = floor(e + 0.5)
		end
	end)
	f:SetScript("OnEnter", function()
		GameTooltip:SetOwner(this, "ANCHOR_RIGHT")
		GameTooltip:SetText("InFlight")
		GameTooltip:AddLine(" Shift left-click and drag to move.")
		GameTooltip:AddLine(" Right-click for menu.")
		GameTooltip:Show()
	end)
	f:SetScript("OnLeave", function() GameTooltip:Hide() end)
	
	f.bar = CreateFrame("StatusBar", nil, f)
	f.spark = f.bar:CreateTexture(nil, "OVERLAY")
	f.locText = f.bar:CreateFontString(nil, "OVERLAY")
	f.timeText = f.bar:CreateFontString(nil, "OVERLAY")
	f.bar:ClearAllPoints()
	f.bar:SetPoint("TOPLEFT", 5, -5)
	f.bar:SetPoint("BOTTOMRIGHT", -5, 5)
	
	self:LoadOptions()
	self.dewdrop:Register( f,
		'children', function() self.dewdrop:FeedAceOptionsTable(self.opts) end,
		'cursorX', true, 'cursorY', true )
		
	self.frame = f
	self:UpdateLook()

end

function InFlight:UpdateLook()
	local db = self.db.profile
	local font, _ = GameFontHighlightSmall:GetFont()
	local f = self.frame
	local inset = 5
	
	f:SetWidth(db.width+10)
	f:SetHeight(db.height+10)
	if db.border == "Textured" then inset = 3 end
	f:SetBackdrop({ 
		bgFile = self.surface:Fetch(db.texture), 
		edgeFile = (db.border=="Default" and "Interface\\Tooltips\\UI-Tooltip-Border") or "", edgeSize = 16,
		insets = {left = inset, right = inset, top = inset, bottom = inset},
	})
	f:SetBackdropColor(db.backcolor.r, db.backcolor.g, db.backcolor.b, 0.6)
	f:SetBackdropBorderColor(db.bordercolor.r, db.bordercolor.g, db.bordercolor.b, 0.8)
	
	f.bar:SetStatusBarTexture(self.surface:Fetch(db.texture))
	
	f.spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
	f.spark:SetWidth(16)
	f.spark:SetHeight(db.height*2.44)
	f.spark:SetBlendMode("ADD")
	
	f.timeText:SetFont(font, db.fontsize)
	f.timeText:ClearAllPoints()
	f.locText:SetFont(font, db.fontsize)
	f.locText:ClearAllPoints()
	f.timeText:SetShadowColor(0.1, 0.1, 0.1)
	f.timeText:SetShadowOffset(1,-1)
	f.locText:SetShadowColor(0.1, 0.1, 0.1)
	f.locText:SetShadowOffset(1,-1)
	f.timeText:SetTextColor(db.fontcolor.r, db.fontcolor.g, db.fontcolor.b)
	f.locText:SetTextColor(db.fontcolor.r, db.fontcolor.g, db.fontcolor.b)
	
	if db.inline then
		f.timeText:SetJustifyH("RIGHT")
		f.timeText:SetWidth(f:GetWidth())
		f.timeText:SetPoint("RIGHT", f, "RIGHT", -8, 0)

		f.locText:SetJustifyH("LEFT")
		f.locText:SetJustifyV("CENTER")
		f.locText:SetWidth(f:GetWidth()*0.7)
		f.locText:SetPoint("LEFT", f, "LEFT", 8, 0)	
	else
		f.timeText:SetJustifyH("CENTER")
		f.timeText:SetWidth(f:GetWidth())
		f.timeText:SetPoint("CENTER", f, "CENTER", 0, 0)

		f.locText:SetJustifyH("CENTER")
		f.locText:SetWidth(max(200, f:GetWidth()*1.2))
		f.locText:SetPoint("BOTTOM", f, "TOP", 0, (db.border~="Default" and -3) or -1)
	end	
end


function InFlight:LoadOptions()
	if self.opts then return end
	local db = self.db.profile
	self.opts = {
		type = "group",
		args = {
			header = {
				type = "header",
				name = "InFlight",
				order = 1,
			},
			frame = {
				type = "group",
				name = "Appearance",
				desc = "Change the way the statusbar look and behave.",
				order = 2,
				args = {
					inline = {
						type = "toggle",
						name = "Inline Mode",
						desc = "Shows only destination/time inside the statusbar",
						set = function(v) db.inline = v self:UpdateLook() 
							if not self.destination then return end
							if db.inline then
								self.frame.locText:SetText(self.destination)
							else
								self.frame.locText:SetText(string.format("%s --> %s", self.source, self.destination))
							end
						end,
						get = function() return db.inline end,
						order = 1,
					},
					barfill = {
						type = "text",
						name = "Bar Fill",
						desc = "Set the fill type of the statusbar",
						set = function(v) db.fill = v self:UpdateLook() end,
						get = function() return db.fill end,
						validate = { "Fill up", "Deplete", },
						order = 2,
					},
					height = {
						type = "range",
						name = "Height",
						desc = "Height",
						set = function(v) db.height = v self:UpdateLook() end,
						get = function() return db.height end,
						min = 4, max = 100, step = 2,
						order = 3,
					},
					width = {
						type = "range",
						name = "Width",
						desc = "Width",
						set = function(v) db.width = v self:UpdateLook() end,
						get = function() return db.width end,
						min = 100, max = 600, step = 5,
						order = 4,
					},
					fontsize = {
						type = "range",
						name = "Font Size",
						desc = "Font Size",
						set = function(v) db.fontsize = v self:UpdateLook() end,
						get = function() return db.fontsize end,
						min = 4, max = 40, step = 1,
						order = 5,
					},
					border = {
						type = "text",
						name = "Border",
						desc = "Border Options",
						set = function(v) db.border = v self:UpdateLook() end,
						get = function() return db.border end,
						validate = { "None", "Default", "Textured", },
						order = 6,
					},
					bartexture = {
						type = "text", 
						name = "Bar Texture",
						desc = "Set the texture of the statusbar",
						get = function() return db.texture end,
						set = function(v) 
							db.texture = v 
							self.frame.bar:SetStatusBarTexture(self.surface:Fetch(db.texture)) 
							self:UpdateLook()
						end,
						validate = self.surface:List(),
						order = 7,
					},
					barcolor = {
						type = "color",
						name = "Bar Color",
						desc = "Set the color of the fill bar.",
						get = function() return db.barcolor.r, db.barcolor.g, db.barcolor.b end,
						set = function(r,g,b) 
							db.barcolor = {r=r, g=g, b=b} 
							if self.endTime then 
								self.frame.bar:SetStatusBarColor(r, g, b) 
							end
						end,
						order = 8,
					},
					unknowncolor = {
						type = "color",
						name = "Unknown Color",
						desc = "Set the color of the fill bar for unknown flights.",
						get = function() return db.unknowncolor.r, db.unknowncolor.g, db.unknowncolor.b end,
						set = function(r,g,b) 
							db.unknowncolor = {r=r, g=g, b=b} 
							if not self.endTime then
								self.frame.bar:SetStatusBarColor(r, g, b) 
							end
						end,
						order = 9,
					},
					bordercolor = {
						type = "color",
						name = "Border Color",
						desc = "Sets the color of the statusbar's border.",
						get = function() return db.bordercolor.r,db.bordercolor.g,db.bordercolor.b end,
						set = function(r,g,b) db.bordercolor = {r=r,g=g,b=b} self:UpdateLook() end,
						order = 10,
					},
					backcolor = {
						type = "color",
						name = "Background Color", 
						desc = "Sets the color of the statusbar's background.",
						get = function() return db.backcolor.r, db.backcolor.g, db.backcolor.b end,
						set = function(r,g,b) db.backcolor = {r=r,g=g,b=b} self:UpdateLook() end,
						order = 11,
					},
					fontcolor = {
						type = "color",
						name = "Font Color", 
						desc = "Sets the color of the text.",
						get = function() return db.fontcolor.r, db.fontcolor.g, db.fontcolor.b end,
						set = function(r,g,b) db.fontcolor = {r=r,g=g,b=b} self:UpdateLook() end,
						order = 12,
					},
				},
			},
			dismount = {
				type = "toggle",
				order = 4,
				name = "Auto-dismount",
				desc = "Auto-dismount when talking to a flightmaster.",
				set = function(v) db.dismount = v end,
				get = function() return db.dismount end,
			},
			confirm = {
				type = "toggle",
				order = 5,
				name = "Confirm flight",
				desc = "Confirm the flight before taking it.",
				set = function(v) db.confirmflight = v end,
				get = function() return db.confirmflight end,
			},
		},
	}
	self.dewdrop = AceLibrary("Dewdrop-2.0")
	self:RegisterEvent("Surface_SetGlobal", "UpdateLook")
end

