
InFlight = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.1", "AceDB-2.0", "AceConsole-2.0", "Metrognome-2.0")
local L = AceLibrary("AceLocale-2.2"):new("InFlight")
InFlight.surface = AceLibrary("Surface-1.0")

L:RegisterTranslations("enUS", function() return {
	["Nighthaven"] = true,				--Nighthaven, Moonglade
	["I'd like to fly to (.+)."] = true,	--Druid gossip option
	["Plaguewood"] = true,				--Plaguewood, Eastern Plaguelands
	["Take me to (.+)."] = true,			--EPL Plaguewood Tower flight
	
	-- Shatter Point for Alliance
	["Expedition Point"] = true,			--Expedition Point
	["Hellfire Peninsula"] = true,		--Another for Shatter Point (aka Honor Point)
	["Shatter Point"] = true,
	["Honor Point"] = true,
	["Send me to (.+)!"] = true,			--Shatter Point/Honor Point flightpath gossip option (Alliance)
} end)

L:RegisterTranslations("koKR", function() return {
	["Nighthaven"] = "나이트헤이븐",
	["I'd like to fly to (.+)."] = "(.+)|1으로;로; 가고 싶습니다.",
	["Plaguewood"] = "역병의 숲",
	["Take me to (.+)."] = "(.+)|1으로;로; 갑니다.",
} end)

local vars

function InFlight:OnInitialize()
	self:RegisterEvent("TAXIMAP_OPENED")
	self:RegisterDB("InFlightDB")
	self:RegisterDefaults("profile", {
		confirmflight = false,
		dismount = true, 
		fill = "Fill up",
		inline = false,
		border = "Default",
		height = 14,
		width = 230,
		fontsize = 12,
		texture = "smooth",
		barcolor = { r = 0.5, g = 0.5, b = 0.8, },
		unknowncolor = { r = 0.2, g = 0.2, b = 0.4, },
		bordercolor = { r = 0.3, g = 0.3, b = 0.3, },
		backcolor = { r = 0.1, g = 0.1, b = 0.1, },
		fontcolor = { r = 1.0, g = 1.0, b = 1.0, },
	} )
	self:RegisterChatCommand({ "/inflight"}, {
		type = "execute", name = "InFlight", desc = "InFlight",
		func = function() 
			self:LoadOptions() 
			self.dewdrop:Open( UIParent, 'children', function() self.dewdrop:FeedAceOptionsTable(self.opts) end, 'cursorX', true, 'cursorY', true ) 
		end,
	})
	self.surface:Register("Smooth", "Interface\\AddOns\\InFlight\\smooth")
	
	if Cartographer_Notes then
		self:RegisterEvent("AceEvent_FullyInitialized", "RegisterCarto")
	end
	
	self:SecureHook("GossipTitleButton_OnClick", "TakeMiscFlight")
end

-- Cartographer: load POI data and register database
function InFlight:RegisterCarto()
	local db = self.db.profile
	if db.dataSet ~= 2 then 
		self:LoadPOIData() 
		db.dataSet = 2
	end
	if not db.poidata[UnitFactionGroup("player")] then 
		db.poidata[UnitFactionGroup("player")] = { } 
	end
	Cartographer_Notes:RegisterIcon("TaxiIcon", { 
		text = "Flight Master", path = "Interface\\TaxiFrame\\UI-Taxi-Icon-Green", 
		width = 12, height = 12, alpha = 0.75, } )
	Cartographer_Notes:RegisterIcon("TaxiIcon2", { 
		text = "Flight Master", path = "Interface\\TaxiFrame\\UI-Taxi-Icon-Yellow", 
		width = 10, height = 10, alpha = 0.75, } )
	Cartographer_Notes:RegisterNotesDatabase("InFlight", db.poidata[UnitFactionGroup("player")], self)
	Cartographer_Notes:RefreshMap()
end

-- setup only when the taxi map is opened the first time
function InFlight:TAXIMAP_OPENED(misc)
	if not self.initbulk then
		if not InFlightVars then self:LoadDefaults() end
		if not self.db.profile.TBCLoaded then
			self:LoadTBCData()
			self.db.profile.TBCLoaded = 1
		end
		vars = InFlightVars[UnitFactionGroup("player")]
	
		self:Hook("TakeTaxiNode", true)
		self:SecureHook("TaxiNodeOnButtonEnter")
		
		-- function hooks to detect if a user took a summon
		self:SecureHook("AcceptBattlefieldPort", function(index,accept) if accept then self.porttaken = 1 end end)
		self:SecureHook("ConfirmSummon", function() self.porttaken = 1 end)
		
		self:RegisterMetro(self.name, self.FlightTime, 0.2, self)
		self.initbulk = 1
	end
	if not misc then
		for i = 1, NumTaxiNodes(), 1 do
			if TaxiNodeGetType(i) == "CURRENT" then
				self.source = self:ShortenName(TaxiNodeName(i))
				break
			end
		end
	end
	if self.db.profile.dismount then 
		Dismount() 
	end
	if Cartographer_Notes then self:NoteIt(misc) end
end

-- add time info on the taxi map tooltips
function InFlight:TaxiNodeOnButtonEnter(button)
	
	local index = button:GetID()
	if TaxiNodeGetType(index) == "REACHABLE" then
		local destination = self:ShortenName(TaxiNodeName(index))

		if self.source and vars[self.source] and vars[self.source][destination] then
			local t = vars[self.source][destination]
			if t == 0 then
				GameTooltip:AddLine("Time: -:--", 0.8, 0.8, 0.8)
			else
				GameTooltip:AddLine(string.format("Time: %s", self:FormatTime(t)), 1, 1, 1)
			end
		else
			GameTooltip:AddLine("Time: -:--", 0.8, 0.8, 0.8)
		end
		GameTooltip:Show()
	end
end

-- main hook to determine when a flight is taken
function InFlight:TakeTaxiNode(slot)
	if TaxiNodeGetType(slot) == "REACHABLE" then
		
		-- check to see if there's data for the source/destination pair
		self.destination = self:ShortenName(TaxiNodeName(slot))
		if vars[self.source] and 
		   vars[self.source][self.destination] and 
		   vars[self.source][self.destination] ~= 0 then
			self.endTime = vars[self.source][self.destination]
		else
			self.endTime = nil
		end

		-- confirm or start the flight
		if self.db.profile.confirmflight then
			self:ConfirmFlight(slot, self.destination, self.endTime)
		else
			self:StartTimer(slot)
		end
	end
end

-- setup the timer bar then begin timing
function InFlight:StartTimer(slot)
	self.totalTime = 0
	if not self.frame then self:CreateBar() end
	local db, f = self.db.profile, self.frame
	if db.inline then
		f.locText:SetText(self.destination)
	else
		f.locText:SetText(string.format("%s --> %s", self.source, self.destination))
	end
	
	-- start the timers and setup statusbar
	if self.endTime then
		f.bar:SetMinMaxValues(0, self.endTime)
		if db.fill == "Deplete" then
			f.bar:SetValue(self.endTime)
		else
			f.bar:SetValue(0)
		end
		f.bar:SetStatusBarColor(db.barcolor.r, db.barcolor.g, db.barcolor.b, 0.8)
		f.spark:Show()
	else
		f.bar:SetMinMaxValues(0,1)
		f.bar:SetValue(1)
		f.bar:SetStatusBarColor(db.unknowncolor.r, db.unknowncolor.g, db.unknowncolor.b, 0.8)
		f.spark:Hide()
	end
	self:StartMetro(self.name)
	self.porttaken = nil
	
	self:ScheduleEvent(function() if not self.active then self:StopMetro(self.name) end end, 6, self)
	
	if slot then self.hooks.TakeTaxiNode(slot) end
end

-- in flight onupdate function
function InFlight:FlightTime(difference)
	if not self.active and UnitOnTaxi("player") then self.active = 1 end
	if not self.active then return end
	local f = self.frame
	self.totalTime = self.totalTime + difference
	f:Show()
	if not UnitOnTaxi("player") then -- flight ended
		if not self.porttaken then self:SaveFlightTime() end
		f:Hide()
		self.porttaken = nil
		self.endTime = nil
		self.active = nil
		self:StopMetro(self.name)
		return
	end
	if self.endTime then
		if self.totalTime - 2 > self.endTime then -- in case the flight is longer than expected
			f.bar:SetMinMaxValues(0,1)
			f.bar:SetValue(1)
			f.bar:SetStatusBarColor(self.db.profile.unknowncolor.r, self.db.profile.unknowncolor.g, self.db.profile.unknowncolor.b, 0.8)
			f.spark:Hide()
			self.endTime = nil
		else
			if self.db.profile.fill == "Deplete" then
				f.bar:SetValue(self.endTime - self.totalTime)
				f.spark:SetPoint("CENTER", f.bar, "LEFT", ((self.endTime - self.totalTime) / self.endTime) * f.bar:GetWidth(), 0)
				f.timeText:SetText(string.format("%s / %s", self:FormatTime(self.endTime - self.totalTime), self:FormatTime(self.endTime)))
			else
				f.bar:SetValue(self.totalTime)
				f.spark:SetPoint("CENTER", f.bar, "LEFT", (self.totalTime / self.endTime) * f.bar:GetWidth(), 0)
				f.timeText:SetText(string.format("%s / %s", self:FormatTime(self.totalTime), self:FormatTime(self.endTime)))
			end
		end
	else
		f.timeText:SetText(string.format("Timing - %s", self:FormatTime(self.totalTime)))
	end
end

-- ask user to confirm flight before taking it
function InFlight:ConfirmFlight(slot, location, flighttime)
	local texttime = ""
	if flighttime then
		texttime = string.format("This flight will take %s.", self:FormatTime(flighttime))
	end
	StaticPopupDialogs["INFLIGHT_CONFIRMTAKEOFF"] = {
		text = TEXT(string.format("Are you sure you want to fly to %s? %s", location, texttime)),
		button1 = TEXT(OKAY),
		button2 = TEXT(CANCEL),
		OnAccept = function(data)
			InFlight:StartTimer(data)
		end,
		timeout = 0,
		exclusive = 1,
		hideOnEscape = 1,
	}
	local dialog = StaticPopup_Show("INFLIGHT_CONFIRMTAKEOFF")
	if dialog then dialog.data = slot end
end

-- save data
function InFlight:SaveFlightTime()
	if not vars[self.source] then vars[self.source] = { } end
	vars[self.source][self.destination] = floor(self.totalTime + 0.5)
	self.totalTime = 0
end


-- shorten name to lighten saved vars
function InFlight:ShortenName(name)
	local comma = ", "
	local found = string.find(name, comma)
	if found then name = string.sub(name, 1, found - 1) end
	return name
end

-- simple time format
function InFlight:FormatTime(secs)
	return string.format("%d:%02d", secs / 60, math.fmod(secs, 60))
end

-- record POI location for Cartographer_Notes (you make me work so hard...)
function InFlight:NoteIt(misc)
	if not vars[self.source] then vars[self.source] = { } end
	SetMapToCurrentZone()
	local x,y = GetPlayerMapPosition("player")
	if not Cartographer_Notes:GetNearbyNote(Cartographer:GetCurrentEnglishZoneName(), x, y, 10, "InFlight", true) then
		Cartographer_Notes:SetNote(
			Cartographer:GetCurrentEnglishZoneName(), x, y, (misc and "TaxiIcon2") or "TaxiIcon", "InFlight",
			"title", "Flight Master", "titleR", 0, "titleB", 0 )
	end
end

-- support for druid-only paths and EPL tower paths
function InFlight:TakeMiscFlight()
	local buttonText = this:GetText()
	if buttonText then
		local subzone = GetMinimapZoneText()
		
		-- Druid-only FP in Moonglade
		if subzone == L["Nighthaven"] and strfind(buttonText, L["I'd like to fly to (.+)."]) then
			self.source = subzone
			_,_,self.destination = strfind(buttonText, L["I'd like to fly to (.+)."])
			
		-- Plaguewood tower (PvP objective)	
		elseif subzone == L["Plaguewood"] and strfind(buttonText, L["Take me to (.+)."]) then
			self.source = subzone
			_,_,self.destination = strfind(buttonText, L["Take me to (.+)."])
			
		-- weird Alliance flights in Hellfire
		elseif (subzone == L["Expedition Point"] or subzone == L["Hellfire Peninsula"] or subzone == L["Shatter Point"]) 
		       and strfind(buttonText, L["Send me to (.+)!"]) then
			self.source = (subzone == L["Hellfire Peninsula"] and L["Honor Point"]) or subzone
			_,_,self.destination = strfind(buttonText, L["Send me to (.+)!"])
		else
			return
		end
		self:TAXIMAP_OPENED(true)
		if vars[self.source] and vars[self.source][self.destination] then
			self.endTime = vars[self.source][self.destination]
		else
			self.endTime = nil
		end
		self:StartTimer()
	end
end
